package com.jeeplus.test.reimbursementsys.mapper;

import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.jeeplus.test.reimbursementProject.domain.dto.ProjectReimbursementDTO;
import com.jeeplus.test.reimbursementsys.domain.dto.ReimbursementDTO;
import org.apache.ibatis.annotations.Param;

import java.util.List;

/**
 * @author li-peike
 * @version 1.0.0
 * @ClassName ReimbursementSysMapper.java
 * @Description TODO
 * @createTime 2022年05月06日 10:40:00
 */
public interface ReimbursementSysMapper  extends BaseMapper<ReimbursementDTO> {
    /**
     * 根据条件查询数据信息
     * @param page
     * @param reimbursement
     * @return
     */
    IPage<ReimbursementDTO> findList(Page<ReimbursementDTO> page, @Param("reimbursement")ReimbursementDTO reimbursement);
    /**
     * 根据条件查询数据信息
     * @param page
     * @param reimbursement
     * @return
     */
    IPage<ReimbursementDTO> queryAllList(Page<ReimbursementDTO> page, @Param("reimbursement")ReimbursementDTO reimbursement);


    List<ReimbursementDTO> getInvoiceList(ReimbursementDTO reimbursement);

    /**
     * 根据发票id查询业务具体信息
     * @param id
     * @return
     */
    ReimbursementDTO queryBusinessByInvoiceId(String id);

    /**
     * 根据父id查询已收款的发票信息的首款额度总和
     * @param parentId
     * @return
     */
    String getReceiptInvoiceIncomeByReimbursementId(String parentId);

    /**
     * 根据业务id查询报销比例
     * @param id
     * @return
     */
    String getReimbursementRatioByReimbursementId(String id);

    /**
     * 根据父节点id查询发票信息
     * @param parentIdList
     * @return
     */
    List<ReimbursementDTO> getInvoiceByParentIdList(@Param("parentIdList")List<String> parentIdList);

    /**
     * 查询是否存在需要保存的发票信息
     * @param invoiceNumberList
     * @return
     */
    List<String> selectOldInvoiceNumber(@Param("invoiceNumberList") List<String> invoiceNumberList);

    /**
     * 查询是否存在已有的业务编号信息
     * @param businessCodeList
     * @return
     */
    List<String> selectOldBusinessCode(@Param("businessCodeList") List<String> businessCodeList);

    /**
     * 查询是否存在已有的业务编号信息（业务编码和年份）
     * @param businessCodeList
     * @return
     */
    List<ReimbursementDTO> selectOldBusinessCodeByInfo(@Param("businessCodeList") List<String> businessCodeList,@Param("year") String year);

    /**
     * 保存业务编号信息
     * @param businessCodeList
     */
    void insertBusinessCodeList(@Param("businessCodeList")List<ReimbursementDTO> businessCodeList);

    /**
     * 保存发票编号数据信息
     * @param invoiceInfoList
     */
    void insertInvoiceList(@Param("invoiceInfoList")List<ReimbursementDTO> invoiceInfoList);

    /**
     * 查询业务信息
     * @param id
     * @return
     */
    ReimbursementDTO queryBusinessById(String id);

    /**
     * 查询发票信息
     * @param id
     * @return
     */
    ReimbursementDTO queryInvoiceById(String id);

    /**
     * 保存业务编码
     * @param reimbursement
     * @return
     */
    Integer saveBusiness(ReimbursementDTO reimbursement);

    /**
     * 保存发票收款时间
     * @param reimbursement
     * @return
     */
    Integer saveInvoice(ReimbursementDTO reimbursement);

    /**
     * 修改发票信息
     * @param reimbursement
     * @return
     */
    Integer modifyInvoice(ReimbursementDTO reimbursement);

    /**
     * 根据业务编码查询业务信息
     * @param reimbursement
     * @return
     */
    ReimbursementDTO getInfoByBusinessCode(ReimbursementDTO reimbursement);

    /**
     * 批量修改发票信息
     * @param invoiceInfoList
     */
    void updateInvoiceList(@Param("invoiceInfoList")List<ReimbursementDTO> invoiceInfoList);

    /**
     * 根据id删除对应业务信息（物理删除）
     * @param id
     */
    void deleteBusinessById(String id);

    /**
     * 删除业务信息（逻辑删除）
     * @param idList
     */
    void removeBusinessByIds(@Param("idList")List<String> idList);


    /**
     * 删除发票信息（逻辑删除）
     * @param idList
     */
    void removeBusinessByParentIds(@Param("idList")List<String> idList);

    /**
     * 删除发票信息（逻辑删除）
     * @param idList
     */
    void removeInvoiceByIds(@Param("idList")List<String> idList);

    /**
     * 修改业务报销比例
     * @param reimbursement
     */
    void updateReimbursementRatio(ReimbursementDTO reimbursement);

    /**
     * 根据发票id查询是否已经填写收款时间
     * @param idList
     * @return
     */
    List<ReimbursementDTO> getGatheringTimeInvoiceList(@Param("idList") List<String> idList);

    /**
     * 根据该条发票信息查询同业务编号下的没有收款的发票信息
     * @param reimbursement
     * @return
     */
    List<ReimbursementDTO> getNotGatheringTimeInvoice(ReimbursementDTO reimbursement);

    /**
     * 根据该条发票信息查询同业务编号下的没有收款的发票信息
     * @param reimbursement
     * @return
     */
    List<ReimbursementDTO> getNotGatheringTimeInvoiceBybusinessCodeId(ReimbursementDTO reimbursement);

    void updateGatheringTime(@Param("idList") List<String> idList, @Param("reimbursement") ReimbursementDTO reimbursement);

    /**
     * 根据id修改业务信息的收款状态
     * @param id
     */
    void updateGatheringStatusById(String id);

    /**
     * 查询最后一个随机业务编码的编码序号
     * @return
     */
    String getLastRandomBusinessCodeIndex(String year);

    /**
     * 查询该发票信息下是否存在有效的同级数据信息
     * @param id
     */
    String getValidBusinessIdByInvoiceId(String id);
    /**
     * 删除业务信息（逻辑删除）
     * @param id
     */
    void removeBusinessById(String id);

    /**
     * 根据发票id查询发票信息
     * @param idList
     * @return
     */
    List<ReimbursementDTO> getInvoiceByIdList(@Param("idList") List<String> idList);

    /**
     * 根据id删除对应业务信息（物理删除）
     * @param idList
     */
    void deleteBusinessByIdList(@Param("idList")List<String> idList);

    /**
     * 修改业务信息
     * @param reimbursement
     */
    int updateById(ReimbursementDTO reimbursement);


    /**
     * 根据父id查询已收款的发票信息的首款额度总和
     * @param parentId
     * @return
     */
    String getReceiptInvoiceIncomeByBindingReimbursementId(@Param("parentId")String parentId,@Param("reimbursementAddress")String reimbursementAddress);

    /**
     * 根据业务id查询报销比例
     * @param id
     * @return
     */
    String getReimbursementRatioByBindingReimbursementId(@Param("id")String id,@Param("reimbursementAddress")String reimbursementAddress);
}
